import math

def average(_list):
    return sum(_list)/(len(_list))

def stddev(_list):
    avg = average(_list)
    total = 0
    for l in _list:
        total += (l-avg)**2
    
    avg_sqred_diff = total/len(_list)

    return math.sqrt(avg_sqred_diff)

def zscore(_observered, _mean, _standard_deviation):
    return (_observered-_mean)/_standard_deviation

def corolation(x, y):
    x_hat = average(x)
    y_hat = average(y)

    x_sd = stddev(x)
    y_sd = stddev(y)

    zxzy = []
    for i in range(len(x)):
        zxzy.append(zscore(x[i], x_hat, x_sd) * zscore(y[i], y_hat, y_sd))
    
    return average(zxzy)





from howitzer.util.stats import *

import unittest


class TestFoltillaStats(unittest.TestCase):
    def setUp(self):
        self.test_list = [-5, 1, 8, 7, 2]

    def test_average(self):
        self.assertEqual(average(self.test_list), 2.6, "average is incorrect")

    def test_stddev(self):
        self.assertAlmostEqual(stddev(self.test_list),
                               4.6733, 4, "average is incorrect")

    def test_zscore(self):
        self.assertAlmostEqual(zscore(-5, 2.6, 4.6733), -
                               1.6263, 4, "zscore is incorrect")

    def test_corolation(self):
        x = [1, 3, 4, 5, 7]
        y = [5, 9, 7, 1, 13]
        self.assertAlmostEqual(corolation(x, y), 0.4, 1,
                               "corolation is incorrect")



from datetime import datetime, timedelta
from howitzer.util.date import shortStringFormat
from os.path import join
import dateutil.parser as dateparse
import ujson as json
import math


class Candle:
    def __init__(self, currentData: dict, previousClose: float = None):
        self.time = round(dateparse.parse(
            currentData["time"]).timestamp() * 1000)
        self.low = currentData["low"]
        self.high = currentData["high"]
        self.open = currentData["open"]
        self.close = currentData["close"]
        self.volume = currentData["volume"]
        self.percent = 100 * (self.close - self.open) / self.open
        self.diff = self.close - self.open
        self.range = self.high - self.low
        self.green = self.close > self.open
        self.red = self.close < self.open
        self.head = self.high - max(self.close, self.open)
        self.tail = min(self.open, self.close) - self.low
        self.body = max(self.close, self.open) - min(self.close, self.open)
        self.twap = (self.close + self.high + self.low) / 3
        self.tr = self.range
        if previousClose is not None:
            self.tr = max(self.range, abs(self.high-previousClose),
                          abs(self.low-previousClose))


class Chart:
    def __init__(this, rawCandleData: list):
        this.candles = []
        number_of_candles_to_parse = len(rawCandleData)
        for i in range(number_of_candles_to_parse):
            if i < number_of_candles_to_parse - 1:
                this.candles.append(
                    Candle(rawCandleData[i], previousClose=rawCandleData[i+1]["close"]))
            if i == number_of_candles_to_parse - 1:
                this.candles.append(Candle(rawCandleData[i]))


def chartFromDataFiles(pathToDataFolder: str, startDate: datetime, endDate: datetime):
    def BLANK_CANDLE():
        return {"time": None, "low": math.inf, "high": -math.inf, "volume": 0}
        # return [None, math.inf, - math.inf, None, None, 0]

    stopTime = endDate.timestamp()
    dailyRawCandles = []
    # todo: add days to queu and tehn deque on multiple threads to get data, sort after the fact
    while(startDate.timestamp() <= stopTime):
        tempDaily = BLANK_CANDLE()
        nextDay = startDate + timedelta(days=1)

        targetFileName = shortStringFormat(startDate) + ".json"
        targetFilePath = join(pathToDataFolder, targetFileName)
        f = open(targetFilePath)

        data = json.load(f)
        # daily logic
        tempDaily["time"] = data[0]["time"]
        tempDaily["open"] = data[0]["open"]
        for minute in data:
            candleTime = dateparse.parse(minute["time"])
            if candleTime.timestamp() < nextDay.timestamp():
                tempDaily["volume"] += minute["volume"]
                tempDaily["close"] = minute["close"]
                tempDaily["high"] = max(tempDaily["high"], minute["high"])
                tempDaily["low"] = min(tempDaily["low"], minute["low"])

        dailyRawCandles.append(tempDaily)
        startDate = nextDay
        f.close()

    dailyRawCandles.reverse()
    return {
        "daily": Chart(dailyRawCandles)
    }



from __future__ import print_function, division
from torch.utils.data import Dataset, DataLoader
import pandas as pd
import torch
import os
__dirname = os.path.dirname(__file__)


def getAllGames():
    return pd.read_csv(os.path.join(__dirname,'files/nfl-all-games.csv'), index_col="index")

def getAllGameStats():
    return pd.read_csv(os.path.join(__dirname,'files/nfl-all-team-stats.csv'), index_col="index")

def getRolling3GameAverage():
    return pd.read_csv(os.path.join(__dirname,'files/machiene-readable-3-week-look-back.csv'))

class NflGameDataset(Dataset):
    """NFL Game Result dataset"""
    
    def __init__(self, csv_file="", transforms=None):
        """
        Args:
            csv_file (string): Path to the csv file containing the results data
        """
        
        self.game_frame = pd.read_csv(csv_file)
        self.transforms = transforms
        
    def __len__(self):
         return len(self.game_frame)
        
    def __getitem__(self, idx):
        if torch.is_tensor(idx):
            idx = idx.tolist()
                
        stats = self.game_frame.iloc[idx, :-1]
        result = self.game_frame.iloc[idx, -1:]
            
        sample = {"result":result, "stats":stats}
            
        if self.transforms:
            sample = self.transforms(sample)
            
        return sample


from howitzer.util.date import *

from datetime import datetime, timezone
import unittest


class TestFoltillaDateStrings(unittest.TestCase):
    def setUp(this):
        this.test_date = datetime(2021, 1, 1)

    def test_shortStringFormat(this):
        this.assertEqual(shortStringFormat(this.test_date), "1Jan2021")

    def test_shortMonthString_Jan(this):
        this.assertEqual(shortMonthString(1), "Jan")

    def test_shortMonthString_Feb(this):
        this.assertEqual(shortMonthString(2), "Feb")

    def test_shortMonthString_Mar(this):
        this.assertEqual(shortMonthString(3), "Mar")

    def test_shortMonthString_Apr(this):
        this.assertEqual(shortMonthString(4), "Apr")

    def test_shortMonthString_May(this):
        this.assertEqual(shortMonthString(5), "May")

    def test_shortMonthString_Jun(this):
        this.assertEqual(shortMonthString(6), "Jun")

    def test_shortMonthString_Jul(this):
        this.assertEqual(shortMonthString(7), "Jul")

    def test_shortMonthString_Aug(this):
        this.assertEqual(shortMonthString(8), "Aug")

    def test_shortMonthString_Sep(this):
        this.assertEqual(shortMonthString(9), "Sep")

    def test_shortMonthString_Oct(this):
        this.assertEqual(shortMonthString(10), "Oct")

    def test_shortMonthString_Nov(this):
        this.assertEqual(shortMonthString(11), "Nov")

    def test_shortMonthString_Dec(this):
        this.assertEqual(shortMonthString(12), "Dec")



from howitzer.util.trading import *

from datetime import datetime, timezone
import unittest


class TestFoltillaTradingCandle(unittest.TestCase):
    def setUp(self):
        self.test_candle_01_inputs = {"time": "2021-01-01T00:00:00.000Z",
                                      "open": 50, "high": 55, "low": 49, "close": 51, "volume": 10000}
        self.test_candle_01 = Candle(self.test_candle_01_inputs)

    def test_createSimpleCandle_time(self):
        self.assertEqual(self.test_candle_01.time, 1609459200000)

    def test_createSimpleCandle_open(self):
        self.assertEqual(self.test_candle_01.open,
                         self.test_candle_01_inputs["open"])

    def test_createSimpleCandle_high(self):
        self.assertEqual(self.test_candle_01.high,
                         self.test_candle_01_inputs["high"])

    def test_createSimpleCandle_low(self):
        self.assertEqual(self.test_candle_01.low,
                         self.test_candle_01_inputs["low"])

    def test_createSimpleCandle_close(self):
        self.assertEqual(self.test_candle_01.close,
                         self.test_candle_01_inputs["close"])

    def test_createSimpleCandle_volume(self):
        self.assertEqual(self.test_candle_01.volume,
                         self.test_candle_01_inputs["volume"])

    def test_createSimpleCandle_percent(self):
        self.assertEqual(self.test_candle_01.percent,  2)

    def test_createSimpleCandle_diff(self):
        self.assertEqual(self.test_candle_01.diff,  1)

    def test_createSimpleCandle_range(self):
        self.assertEqual(self.test_candle_01.range,  6)

    def test_createSimpleCandle_green(self):
        self.assertEqual(self.test_candle_01.green,  True)

    def test_createSimpleCandle_red(self):
        self.assertEqual(self.test_candle_01.red,  False)

    def test_createSimpleCandle_head(self):
        self.assertEqual(self.test_candle_01.head,  4)

    def test_createSimpleCandle_tail(self):
        self.assertEqual(self.test_candle_01.tail,  1)

    def test_createSimpleCandle_body(self):
        self.assertEqual(self.test_candle_01.body,  1)

    def test_createSimpleCandle_twap(self):
        self.assertAlmostEqual(self.test_candle_01.twap,  51.6666667)

    def test_createSimpleCandle_tr_empty(self):
        self.assertEqual(self.test_candle_01.tr,  6)

    def test_createSimpleCandle_twap_with_data(self):
        c = Candle(self.test_candle_01_inputs, previousClose=48)
        self.assertEqual(c.tr,  7)


class TestFoltillaTradingChart(unittest.TestCase):
    def setUp(self):
        self.test_chart_input = [{"time": "2021-01-01T05:00:00.000Z",
                                 "open": 50, "high": 55, "low": 49, "close": 51, "volume": 10000}]
        self.test_chart = Chart(self.test_chart_input)

    def test_chartMakesSingleCandle(self):
        self.assertEqual(len(self.test_chart.candles), 1)



def TestMoneyLine(inputs, outputs, model):
    stats = {"correct": 0, "total": len(inputs), "accuracy": 0, "taken": 0}
    for i in range(len(inputs)):
        take_bet, guess = model(inputs[i])
        if take_bet:
            stats["taken"] += 1
            if guess is outputs[i]:
                stats["correct"] += 1

    stats["accuracy"] = stats["correct"]/stats["taken"]
    return stats

def SimulateMoneyLine(account_size=1000, bet_size=0.1, inputs=[], outputs=[], model=None, static_roi=0.8):
    balance = account_size
    bet_history = []
    bet_history.append(1000)
    for i in range(len(inputs)):
        take_bet, guess = model(inputs[i])
        if take_bet:
            if guess is outputs[i]:
                balance += (balance * bet_size) * static_roi
            else:
                balance -= (balance * bet_size)
            bet_history.append(balance)
    return bet_history


from datetime import datetime


def shortMonthString(month: int):
    shortMonthStrings = [
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec",
    ]
    return shortMonthStrings[month-1]


def shortStringFormat(date: datetime):
    return str(date.day) + shortMonthString(date.month) + str(date.year)



def kellyCriterion(_return_including_wager, _porbality_of_win):
    if _return_including_wager <= 1.0:
        return 0
    top = _return_including_wager *_porbality_of_win -1
    bottom = _return_including_wager - 1
    return top / bottom


#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    readme = fh.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = []

test_requirements = []

setup(
    author="Alexander Angelo Ali",
    author_email='downtocrypto@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Tool for analysing sports betting and stock markets.[D[D[D[D[D[Dfinancial markets[2~[2~[Ds.",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='howitzer',
    name='howitzer',
    packages=find_packages(include=['howitzer', 'howitzer.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/aaaelite21/howitzer',
    version='0.1.0',
    zip_safe=False,
)
